import json
import logging
from importlib import import_module
from ..tools import JSONMultiEncoder


class JSONFormatter(logging.Formatter):
    json_encoder_cls: json.JSONEncoder | str | None = JSONMultiEncoder()

    def format(self, record):
        entry = {
            "timestamp": record.timestamp,
            "activity.sequence.elapsed": record.__dict__["activity_elapsed"],
            "activity.sequence.id": record.__dict__["activity_id"],
            "activity.sequence.name": record.__dict__["activity_name"],

            "event.message": record.__dict__["event_message"],
            "event.name": record.__dict__["event_name"],
            "event.snapshot": record.__dict__["event_snapshot"],
            "event.tags": record.__dict__["event_tags"],
            "source": record.__dict__["source"],
            "exception": record.exception
        }

        entry["activity.elapsed"] = entry["activity.sequence.elapsed"][0:0 + 1]
        entry["activity.depth"] = len(entry["activity.sequence.id"])
        entry["activity.id"] = entry["activity.sequence.id"][0:0 + 1]
        entry["activity.name"] = entry["activity.sequence.name"][0:0 + 1]

        entry["activity.previous.elapsed"] = entry["activity.sequence.elapsed"][1:1 + 1]
        entry["activity.previous.id"] = entry["activity.sequence.id"][1:1 + 1]
        entry["activity.previous.name"] = entry["activity.sequence.name"][1:1 + 1]

        if isinstance(self.json_encoder_cls, str):
            *module, cls = self.json_encoder_cls.split(".")
            self.json_encoder_cls = getattr(import_module(".".join(module)), cls)

        return json.dumps(entry, sort_keys=False, allow_nan=False, cls=self.json_encoder_cls)
